//=============================================================================
//  MuseScore
//  Linux Music Score Editor
//  $Id:$
//
//  Tuning system plugin
//	Alter note tuning according to selected tuning system
//  To add a tuning system, you need to an element in the combo of tuningSystem.ui
//  Then, you need to define the difference in cents between the degrees of 
//  the "natural midi tuning" and the given tuning system in the tunings array below.
//
//  Copyright (C)2008 Werner Schweer and others
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================

//
// This is ECMAScript code (ECMA-262 aka "Java Script")
//

//This array contains the difference between natural tuning and tuning systems
var tunings = [];
            //  C     C#      Db      D     D#    Eb      E     F     F#      Gb      G     G#      Ab      A     A#    Bb      B 
tunings[0] = [  0,    0,      0,      0,    0,    0,      0,    0,    0,      0,      0,    0,      0,      0,    0,    0,      0  ];  // natural 
tunings[1] = [  -5.9, -15.6,  -15.6,  -2,  -11.7, -11.7,  2,    -7.8, -17.6,  -17.6,  -3.9, -13.7,  -13.7,  0,   -9.8,  -9.8,   3.9 ]; // pythagorician
tunings[2] = [  10.3, -13.7,  -13.7,  3.4, 20.5,  20.5,   -3.4, 13.7, -10.3,  -10.3,  6.8,  -17.1,  -17.1,  0,    17.1, 17.1,   -6.8]; // Quarter-comma meantone
tunings[3] = [  4.9,  -4.9,   -4.9,   8.8,  -1,   -1,     -8.8, 2.9,  -4.9,   -4.9,   6.8,  -2.9,   -2.9,   0,    1,    1,      -6.8]; // Kyrnberger II
tunings[4] = [  9.8,  -7.8,   -7.8,   5.9,  3.9,  3.9,    2,    7.8,  -2,     -2,     3.9,  -5.9,   -5.9,   0,    13.7, 13.7    -3.9]; // Werkmeister IV C
tunings[5] = [  10.3, -5.5,   -5.5,   3.4,  4.1,  4.1,    -3.4, 13.7, -10.3,  -10.3,  6.8,  -0.7,   -0.7,   0,    8.9,  8.9,    -6.8]; // Marpourg
tunings[6] = [  5.9,  -3.9,   -3.9,   2,    0,    0,      -2,   3.9,  -5.9,   -5.9,   3.9,  -2,     -2,     0,    2,    2,      -3.9]; // Tartini Vallotti

//---------------------------------------------------------
//    init
//    this function will be called on startup of mscore
//---------------------------------------------------------

function init()
      {
      // print("test script init");
      }

//-------------------------------------------------------------------
//    run
//    this function will be called when activating the
//    plugin menu entry
//
//    global Variables:
//    pluginPath - contains the plugin path; file separator is "/"
//-------------------------------------------------------------------

function run()
      {
      var loader = new QUiLoader(null);
      var file   = new QFile(pluginPath + "/tuningSystem.ui");
      file.open(QIODevice.OpenMode(QIODevice.ReadOnly, QIODevice.Text));
      form = loader.load(file, null);
      form.buttonBox.accepted.connect(accept);
      form.show();
      }

//---------------------------------------------------------
//    accept
//    called when user presses "Accept" button
//---------------------------------------------------------

function accept()
    {
      //get selected tuning
      var value = form.cbTuning.currentIndex;
      //pick the right tuning in tunings array
      var tuning = tunings[value];
      
      var cursor = new Cursor(curScore);
      curScore.startUndo();
      for (var staff = 0; staff < curScore.staves; ++staff) {
            cursor.staff = staff;
            for (var v = 0; v < 3; v++) {
              cursor.voice = v;
              cursor.rewind();  // set cursor to first chord/rest

              while (!cursor.eos()) {
                    if (cursor.isChord()) {
                          for (var i = 0; i < cursor.chord().notes(); i++) {
                                var note = cursor.chord().note(i);
                                var pitch = note.pitch % 12;
                                var name = note.name;
                                var idx = getIndex(pitch, name);
                                note.tuning = tuning[idx];
                          }
                    }
                    cursor.next();
              }
            }
       }
      curScore.endUndo();
    }

function getIndex(pitch, name){
    var result;
    switch (pitch) {
    	case 0: //C
    	   result = 0;
    	break;
    	case 1: //C# or Db
    	 result = (name[0] == 'c') ? 1:2;   
    	break;
    	case 2: //D
    	   result =3;
    	break;
    	case 3: //D# or Eb
    	   result = (name[0] == 'd') ? 4:5;   
    	break;
    	case 4: //E
    	   result = 6;
    	break;
    	case 5: //F
    	   result = 7;
    	break;
    	case 6: //F# or Gb
    	   result = (name[0] == 'f') ? 8:9;  
    	break;
    	case 7: //G
    	   result = 10;
    	break;
    	case 8: //G# or Ab
    	   result = (name[0] == 'g') ? 11:12;  
    	break;
    	case 9: //A
    	   result = 13;
    	break;
    	case 10: //A# or Bb
    	   result = (name[0] == 'a') ? 14:15;  
    	break;
    	case 11: //B
    	   result = 16;
    	break;
    	default: 
        result =0;
    	break;
  }
  return result;
}


//---------------------------------------------------------
//    menu:  defines were the function will be placed
//           in the MuseScore menu structure
//---------------------------------------------------------

var mscorePlugin = {
      menu: 'Plugins.Tuning Systems',
      init: init,
      run:  run
      };

mscorePlugin;

